from random import sample, randint


def rand_cut(n: int, cnt: int, min_value: int = 0):
    """将 `n` 切成 `cnt` 份 , 其中每一段的分布相同"""
    n -= min_value * cnt
    lis = sorted([randint(1, n) for _ in range(cnt)])
    for i in range(cnt - 1, 0, -1):
        lis[i] -= lis[i - 1] - min_value
    return lis



import subprocess
from enum import IntEnum
from itertools import filterfalse
from time import perf_counter

from rich import print
from rich.console import Console
from rich.table import Table

from .config import *


class MyFile:
    def __init__(self, path: Path, name: str) -> None:
        res = list(filterfalse(lambda x: name != x.stem, path.iterdir()))
        if len(res) > 1:
            raise ValueError(f"Duplicated files {res}.")
        elif len(res) == 0:
            raise ValueError(f"Path {path} doesn't have target file.")
        self.file = res[0]
        self.exec_cmd: str = ""
        self.exec_path = None
        self.stem = self.file.stem
        self._calc_suffix()
        # self._link_to_exec()

    def __repr__(self) -> str:
        return f"{self.file}"

    def _calc_suffix(self):
        file = self.file
        if file.suffix == ".py":
            self.exec_cmd = "python"

    def _link_to_exec(self):
        ext = self.file.suffix
        if ext == ".py":
            self.exec_path = self.file
        elif ext == ".cpp":
            self.exec_path = exec_path / f"{self.file.stem}.exe"
            if not self.exec_path.is_file():
                self.exec_path = None

    def f_in(self, cnt: int):
        return data_auto_in / f"{cnt}.{self.file.stem}.in"

    def f_out(self, cnt: int):
        return data_auto_out / f"{cnt}.{self.file.stem}.out"

    def compile(self, c: Config) -> bool:
        if self.exec_path:
            return False
        file = self.file
        if file.suffix == ".cpp":
            res = subprocess.run(
                f"g++ -std=c++{c.gcc_version} -O2 -Wall {file} -o {exec_path/file.stem}.exe ",
                timeout=c.max_time_limit,
                encoding="utf8",
                capture_output=True,
            )
            e = res.stderr
            if e:
                print(e) if e else ...
            if res.returncode != 0:
                raise ValueError(f"Got err when compiling {file}")
            self.exec_path = exec_path / (file.stem + ".exe")
        else:
            self.exec_path = file
        return True


class Runner(MyFile):
    def run(self, c: Config, input: str = ""):
        if not self.exec_path:
            raise ValueError("Empty exec path.")
        cmd = f"{self.exec_cmd} {self.exec_path}".strip()
        t = perf_counter()
        res = subprocess.run(
            cmd,
            timeout=c.max_time_limit,
            input=input,
            encoding="UTF-8",
            capture_output=True,
        )
        t = perf_counter() - t
        o, e = res.stdout, res.stderr
        if e:
            print(e) if e else ...
        if res.returncode != 0:
            raise ValueError(f"Got err when running {self.exec_path}")
        return (o, t)


class Checker(MyFile):
    def __init__(self, path: Path = Path("."), name: str = "") -> None:
        if not name:
            super().__init__(default_path, "checker")
        else:
            super().__init__(path, name)

    def check(self, input: Path, output: Path, ans: Path, c: Config):
        if self.exec_path is None:
            raise ValueError("Empty exec path.")
        cmd = f"{self.exec_cmd} {self.exec_path} {input} {output} {ans}".strip()
        res = subprocess.run(
            cmd,
            timeout=c.max_time_limit,
            encoding="utf8",
            capture_output=True,
        )
        e, r = res.stderr, res.returncode
        return (r, e)


class Validator(MyFile):
    def __init__(self, path: Path = Path("."), name: str = "") -> None:
        if not name:
            super().__init__(default_path, "validator")
        else:
            super().__init__(path, name)


class Interactor(MyFile):
    def __init__(self, path: Path = Path("."), name: str = "") -> None:
        if not name:
            super().__init__(default_path, "interactor")
        else:
            super().__init__(path, name)


class IO:
    def __init__(self, cnt: int) -> None:
        self.cnt = cnt

    def w_input(self, file: MyFile, o: str):
        with open(file.f_in(self.cnt), "w") as f:
            f.write(o)

    def w_output(self, file: MyFile, o: str):
        with open(file.f_out(self.cnt), "w") as f:
            f.write(o)

    def r_input(self, file: MyFile, o: str):
        with open(file.f_in(self.cnt), "r") as f:
            return f.read()

    def r_output(self, file: MyFile, o: str):
        with open(file.f_out(self.cnt), "r") as f:
            return f.read()


class Status(IntEnum):
    passed = 0
    unpassed = 1
    pe = 2
    sys_fail = 3
    tle = 20
    ignore = 21


def print_run_status(col: list[str], data: dict[str, dict[str, int]]):
    table = Table(
        title="Generation Result", show_header=True, header_style="bold magenta"
    )
    table.add_column("Test data")
    for i in col:
        table.add_column(i)
    for i in data:
        r = data[i]
        lis = [i]
        for j in r:
            e = data[i][j]
            if e == Status.passed:
                lis.append("🟢")
            elif e == Status.unpassed:
                lis.append("🔴")
            elif e == Status.pe:
                lis.append("🔡")
            elif e == Status.tle:
                lis.append("⛔")
            elif e == Status.ignore:
                lis.append("🚫")
            elif e == Status.sys_fail:
                lis.append("❗")
            else:
                lis.append("❓")
        table.add_row(*lis)
    console = Console()
    console.print(table)



import json
import os
import shutil
import zipfile as zip
from dataclasses import asdict
from itertools import chain
from time import perf_counter

import typer
from loguru import logger

from .config import *
from .utils import *


def load_config() -> Config:
    try:
        f = open(pth / "config.json", "r")
        c = json.load(f)
        c = Config(**c)
        if c.std == False:
            raise NotImplementedError()
        return c
    except:
        logger.error("Get error when load config.json.")
        raise typer.Exit(1)


@app.command()
def init():
    """Init a problem with base files."""
    if list(pth.iterdir()) != []:
        logger.error("Folder is not empty.")
        raise typer.Exit(1)
    for i in new_dir:
        os.makedirs(i)
    init_config()
    if typer.confirm("Copy 'testlib.h' ?"):
        shutil.copy(default_path / "testlib.h", pth / "testlib.h")
    l = ["std.cpp", "checker.cpp", "interactor.cpp", "validator.cpp"]
    for i in l:
        shutil.copy(default_path / f"template_{i}", pth / f"{i}")


@app.command()
def run(
    strict: bool = typer.Option(
        False,
        help="If it's enabled, the process will be interrupted when program gets invalid returncode.",
    ),
    save: bool = typer.Option(
        False, help="Run in save folder without generating new testcase."
    ),
    table: bool = typer.Option(
        True, help="Print the table of generation result if enable."
    ),
    rich: bool = typer.Option(
        True, help="Show rich information by rich.(No implement.)"
    ),
):
    """Generate data.(Overwrite)"""
    logger.add(log)
    logger.info("Start.")

    logger.info("Loading config.")
    c = load_config()

    logger.info("Collecting info.")

    std = Runner(pth, c.std)
    chc = Checker(pth, c.checker)
    ine = Interactor(pth, c.interactor)
    vai = Validator(pth, c.validator)

    gen = [Runner(gen_path, i) for i in c.gen_list]
    ac = [Runner(ac_code_path, i) for i in c.accept_list]
    wa = [Runner(wa_code_path, i) for i in c.wrong_list]

    logger.info("Compiling...")
    s = perf_counter()
    [i.compile(c) for i in chain(gen, ac, wa, [std, chc, ine, vai])]
    logger.info(f"Compile all files in {perf_counter()-s:.3} seconds.")

    logger.info("Generating data sets...")
    data_cnt = 1
    data_set: list[Path] = list()
    used_time: dict[tuple[str, str], int | float] = dict()
    for i in gen:
        logger.info(f"Using generator {i}...")
        for _ in range(c.gen_data_num[i.stem]):
            logger.info(f"Test case {data_cnt}.")
            data = IO(data_cnt)
            data_set.append(i.f_in(data_cnt))
            data_cnt += 1
            o = i.run(c)[0]
            # TODO: Validate input
            data.w_input(i, o)
            for j in chain(ac, wa, [std]):
                if j.stem in c.gen_link_code[i.stem]:
                    res = j.run(c, o)
                    data.w_output(j, res[0])
                    used_time[(i.stem, j.stem)] = res[1]
    info: dict[str, dict[str, int]] = dict()
    for n in range(1, data_cnt):
        d = data_set[n - 1]
        r = d.name.split(".")[1]
        info[d.name] = {}
        logger.info(f"Running test case {d.name}")
        for i in ac:
            if i.stem not in c.gen_link_code[r]:
                info[d.name][i.stem] = Status.ignore
                continue
            res = chc.check(d, i.f_out(n), std.f_out(n), c)
            info[d.name][i.stem] = res[0]
            if strict:
                if res[0] != Status.passed:
                    raise ValueError(
                        f"In Test {n}, accept code got error {Status(res[0]).name}. Info: {res[1]}. "
                    )
            if (
                res[0] == Status.passed
                and used_time.get((r, i.stem), 0) >= c.time_limit
            ):
                info[d.name][i.stem] = Status.tle
        for i in wa:
            if i.stem not in c.gen_link_code[r]:
                info[d.name][i.stem] = Status.ignore
                continue
            res = chc.check(d, i.f_out(n), std.f_out(n), c)
            info[d.name][i.stem] = res[0]
            if strict:
                if res[0] == Status.sys_fail:
                    raise ValueError(
                        f"In Test {n}, wrong code got error {Status(res[0]).name}. Info: {res[1]}. "
                    )
            if (
                res[0] == Status.passed
                and used_time.get((r, i.stem), 0) >= c.time_limit
            ):
                info[d.name][i.stem] = Status.tle
    logger.info("Rendering the result...")
    if table:
        print_run_status([i.stem for i in chain(ac, wa)], info)
    logger.info("Finished.")


@app.command("btf")
def beautify_data():
    """Renumber the data sets in save folder(from 1)."""
    raise NotImplementedError()


@app.command("output")
def output_data(spj: str = typer.Argument("", help="The name of checker you need.")):
    """Rename and zip data(in save folder). Zipped file will be in output folder."""
    sav: dict[str, int] = dict()
    cnt = 1
    c = load_config()
    f = zip.ZipFile(temp_path / "data.zip", "w")
    for i in data_save_in.iterdir():
        idx = i.name.split(".")[0]
        if idx in sav:
            raise ValueError("Duplicated input file.")
        sav[idx] = cnt
        f.write(i, f"{cnt}.in")
        cnt += 1
    for i in data_save_out.iterdir():
        [idx, fr] = i.name.split(".")[0:2]
        if fr != "std":
            continue
        if idx not in sav:
            raise ValueError(f"Std output {idx} has not input file.")
        f.write(i, f"{sav[idx]}.out")
        sav.pop(idx)
    if len(sav) != 0:
        raise ValueError(f"Input file {sav.keys()} has not std output.")
    if c.checker:
        chc = Checker(pth, c.checker)
        f.write(chc.file, f"{spj if spj else chc.file.name}")
    shutil.copy(temp_path / "data.zip", outp_path / "data.zip")
    f.close()


@app.command("add")
def add_data(
    lis: list[int] = [],
    all: bool = typer.Option(True, "--all", "-a", help="Add all test cases."),
):
    """Add data from auto to save"""
    num = max([int(i.name.split(".")[0]) for i in list(data_save_in.iterdir())] + [0])
    for i in data_auto_in.iterdir():
        idx = int(i.name.split(".")[0])
        if all or idx in lis:
            new = i.name.split(".")
            new[0] = str(int(new[0]) + num)
            des = data_save_in / ".".join(new)
            old_std = data_auto_out / f"{idx}.std.out"
            new_std = data_save_out / f"{new[0]}.std.out"
            shutil.copy(i, des)
            shutil.copy(old_std, new_std)
            logger.info(f"Move {i.relative_to(pth)} to {des.relative_to(pth)}")
            logger.info(
                f"Move {old_std.relative_to(pth)} to {new_std.relative_to(pth)}"
            )


@app.command()
def init_config():
    """Init config"""
    logger.add(log)
    res = True
    if os.path.isfile(pth / "config.json"):
        res = typer.confirm("Override the current config?")
        if res:
            logger.warning("Config is overwritten.")
        else:
            logger.info("Interrupted.")
            raise typer.Exit()
    if res:
        with open(pth / "config.json", "w") as f:
            json.dump(asdict(Config()), f)



import os
from dataclasses import dataclass, field
from pathlib import Path
from time import gmtime, strftime

import typer

pth: Path = Path(os.getcwd())
app = typer.Typer()
now = strftime(r"%Y-%m-%d", gmtime())
log = pth / "log" / (now + ".log")


@dataclass
class Config:
    """Config"""

    gen_list: list[str] = field(default_factory=list)
    """Generator list"""
    accept_list: list[str] = field(default_factory=list)
    """Accept code list"""
    wrong_list: list[str] = field(default_factory=list)
    """Wrong code list"""
    gen_link_code: dict[str, list[str]] = field(default_factory=dict)
    """{"make":["wa", "ac"]}"""
    gen_data_num: dict[str, int] = field(default_factory=dict)
    """Number of data sets generated by the generator."""
    time_limit: float = 2
    """Time limit for code."""
    max_time_limit: float = 10
    """Time limit for all programs."""
    std: str = "std"
    """Required"""
    checker: str = "checker"
    """Optional. Default strategy is comparing line by line."""
    interactor: str = "interactor"
    validator: str = "validator"
    gcc_version: int = 17


gen_path = pth / "generator"
ac_code_path = pth / "accept"
wa_code_path = pth / "wrong"
exec_path = pth / "exec"
data_auto_in = pth / "data/auto/in"
data_auto_out = pth / "data/auto/out"
data_save_in = pth / "data/save/in"
data_save_out = pth / "data/save/out"
log_path = pth / "log"
temp_path = pth / "temp"
outp_path = pth / "output"

new_dir = [
    gen_path,
    ac_code_path,
    wa_code_path,
    exec_path,
    data_auto_in,
    data_auto_out,
    data_save_in,
    data_save_out,
    log_path,
    temp_path,
    outp_path,
]

file = Path(__file__)
default_path = file.parent / "default"



from pathlib import Path
import typer
from graphviz import Digraph, Graph

from .system import app as sys_app

app = typer.Typer()
app.add_typer(sys_app, name="sys")


@app.command("render")
def render_graph(
    file: Path = typer.Argument(...),
    directed: bool = typer.Option(False, "--directed", "-d", help="Directed graph."),
    ignore: bool = typer.Option(False, "--ignore", "-i", help="Ignore the first line."),
    clean: bool = typer.Option(False, "--clean", "-c", help="Clean the temp file."),
):
    """Render the graph from file by Graphviz."""
    graph = Digraph if directed else Graph
    dot = graph("Graph", "Rendered by Graphviz")
    n = 0
    with open(file, "r") as f:
        for i, j in enumerate(f):
            if ignore and i == 0:
                n = int(j.split()[0])
                continue
            lis = j.split()
            dot.edge(*lis)
    for i in range(1, n+1):
        dot.node(str(i))
    dot.render(file.stem, format="png", cleanup=clean)



"""Compare output and answer line by line."""
import sys

AC = 0
WA = 1
FAIL = 3

if len(sys.argv) != 4:
    exit(FAIL)

print(len(sys.argv))
inp = open(sys.argv[1], "r")
oup = open(sys.argv[2], "r")
ans = open(sys.argv[3], "r")
try:
    for i, j in zip(oup, ans, strict=True):
        if i.strip() == j.strip():
            continue
        exit(WA)
except ValueError as e:
    exit(WA)
except Exception as e:
    exit(FAIL)
exit(AC)



from pathlib import Path
from typing import Iterator
from typing_extensions import Self
from graphviz import Digraph, Graph
from random import randint

type_weight = float | int

class Edge:
    def __init__(self, u: int, v: int, w: type_weight = 0) -> None:
        """Edge initialization

        Args:
            u (int): start
            v (int): end
            w (type_weight, optional): edge weight. Defaults to 0.
        """
        self.u = u
        self.v = v
        self.w = w
    
    def __repr__(self) -> str:
        return f"{self.u} {self.v} {self.w}"


class BaseGraph:
    """A (weakly) connected graph with no self-loop and multiple edge"""
    def __init__(self, n: int, m: int, directed: bool = False, weighed: bool = False) -> None:
        """
        ! DO NOT USE IT DIRECTLY.
        
        Graph initialization. Vertex start from 1.

        Args:   
            n (int): vertex num
            m (int): edge num
            directed (bool, optional): directed graph. Defaults to False.
            weighed (bool, optional): weighted graph. Defaults to False.
            
        Raises:
            ValueError: Improper number of vertex or edge.
        """
        if n > m+1:
            raise ValueError("Improper number of vertex or edge.")
        self.n = n
        self.m = m
        self.weighed = weighed
        self.directed = directed
        self.edgeList: list[Edge] = []

    def _add_edge(self, u: int, v: int, w: type_weight = 0):
        self.edgeList.append(Edge(u, v, w))

    def add_edge(self, u: int, v: int, w: type_weight = 0):
        self._add_edge(u, v, w)

    def iterator_edge(self) -> Iterator[Edge]:
        return self.edgeList.__iter__()
    
    def render(self, path: Path|str = '.'):
        """Render the graph using Graphviz.

        Args:
            path (Path | str, optional): image output path. Defaults to '.'.

        Raises:
            ValueError: limit the size of the graph.
        """
        if self.n > 100 or self.m > 200:
            raise ValueError("The graph is too big.")
        if type(path) == str:
            path = Path(path)
        graph = Digraph if self.directed else Graph
        dot = graph('Graph', 'Rendered by Graphviz')
        for i in range(1, self.n+1):
            dot.node(str(i))
        if self.weighed:
            for i in self.edgeList:
                dot.edge(str(i.u), str(i.v), str(i.w))
        else:
            for i in self.edgeList:
                dot.edge(str(i.u), str(i.v))
        dot.render(directory=path, format='png')
    
    def gen_weight(self, min_value: int = 0, max_value: int = 1):
        """Generate edge weight between [min_value,max_value]

        Args:
            min_value (int, optional): min value. Defaults to 0.
            max_value (int, optional): max value. Defaults to 1.
            
        Raises:
            ValueError: Graph is already generated.
        """
        if self.weighed:
            for i in self.edgeList:
                i.w = randint(min_value, max_value)
    
    def gen_edge(self):
        if len(self.edgeList) != 0:
            raise ValueError("Graph is already generated.")
        raise NotImplementedError()
            


from random import randint, sample
from typing import Callable


str_sigma = [
    "abcdefghijklmnopqrstuvwxyz",
    "abcdefghijklmnopqrstuvwxyz".upper(),
    "0123456789",
]


class String:
    def __init__(self, _type: str | None = "l", sigma: str | None = None) -> None:
        """
        _type : 字符集
            l -> lower
            u -> upper
            n -> number
            及其组合
        sigma : 自定义字符集 , 与 _type 求并
        """
        s: set[str] = set()
        if _type is not None:
            for i in _type:
                r = 0
                if i == "l":
                    r = 0
                elif i == "u":
                    r = 1
                else:
                    r = 2

                s = s | {_ for _ in str_sigma[r]}
        if sigma is not None:
            s = s | {_ for _ in sigma}
        self.sigma = "".join(s)

    def _random(self, n: int, sigma: str):
        l = len(sigma)
        return "".join([sigma[randint(1, l) - 1] for _ in range(n)])

    def _paral(self, n: int, lim: int | None = None):
        if lim is None:
            lim = randint(1, len(self.sigma))
        lim = min(lim, len(self.sigma))
        lim = max(1, lim)
        s = "".join(sample(self.sigma, lim))
        c = self._random(int(n / 2), s)
        if n % 2 == 1:
            return c + self._random(1, s) + c[::-1]
        else:
            return c + c[::-1]

    def _rand(self, n: int, lim: int | None = None):
        if lim is None:
            lim = randint(1, len(self.sigma))
        lim = min(lim, len(self.sigma))
        lim = max(1, lim)
        s = "".join(sample(self.sigma, lim))
        return self._random(n, s)

    def gen(self, n: int):
        l: list[Callable] = [self._rand, self._paral]
        c = sample(l, 1)[0]
        return c(n)



from typing import Callable
from .base_graph import BaseGraph
from random import randint, sample, random


class Tree(BaseGraph):
    def __init__(self, n: int, weighed: bool = False) -> None:
        """Tree initialization. Root is 1.

        Args:
            n (int): vertex number
            weighed (bool, optional): weighted graph. Defaults to False.
        """
        super().__init__(n, n - 1, True, weighed)

    def _random(self, s: int, t: int):
        """random .

        Args:
            s (int): start
            t (int): end
        """
        if s > t:
            return
        for v in range(s, t + 1):
            u = randint(1, v - 1)
            self.add_edge(u, v)

    def _random_v1(self, r: int, s: int, t: int, dep: int | None = None):
        """random tree.

        Args:
            r (int): root
            s (int): start
            t (int): end
            mxdep (int): max dep
        """
        if s > t:
            return
        if dep is None:
            dep = randint(2, t - s + 2)
        c0: set[tuple[int, int]] = {(r, 1)}
        for i in range(s, t + 1):
            c = sample(c0, 1)[0]
            self.add_edge(c[0], i)
            if c[1] + 1 < dep:
                c0.add((i, c[1] + 1))

    def _random_v2(self, r: int, s: int, t: int, agg: float | None = None):
        """random tree.

        Args:
            r (int): root
            s (int): start
            t (int): end
            agg (float): degree of aggregation
        """
        if s > t:
            return
        if agg is None:
            agg = random()
        c0 = [[] for _ in range(t - s + 2)]
        c0[0].append(r)
        for i in range(s, t + 1):
            j = 0
            while j < t - s + 1 and random() > agg and len(c0[j + 1]) > 0:
                j += 1
            c = sample(c0[j], 1)[0]
            self.add_edge(c, i)
            c0[j + 1].append(i)

    def _binary(self, r: int, s: int, t: int, bal: float | None = None):
        """binary tree.

        Args:
            r (int): root
            s (int): the previous vertex of the starting
            t (int): end
            bal (float): probability of selecting the left side
        """
        if s > t:
            return
        if bal is None:
            bal = random()
        c0 = {r}
        c1 = set()
        for i in range(s, t + 1):
            if (random() <= bal or len(c1) == 0) and len(c0) > 0:
                c = sample(c0, 1)[0]
                c1.add(c)
                c0.remove(c)
            else:
                c = sample(c1, 1)[0]
                c1.remove(c)
            c0.add(i)
            self.add_edge(c, i)

    def gen_edge(self):
        """Tree generation."""

        l: list[Callable] = [self._binary, self._random_v1, self._random_v2]
        c = sample(l, 1)[0]
        c(1, 2, self.n)



from random import randint, sample

from .base_graph import BaseGraph


class Graph(BaseGraph):
    def __init__(
        self, n: int, m: int, directed: bool = False, weighed: bool = False
    ) -> None:
        if n < 0 or m < 0:
            raise ValueError("Improper number of vertex or edge.")
        if n * (n - 1) < m * 2 and directed == False:
            raise ValueError("Improper number of vertex or edge.")
        if n * (n - 1) < m * 2 * 2 and directed == True:
            raise ValueError("Improper number of vertex or edge.")
        super().__init__(n, m, directed, weighed)

    def gen_edge(self):
        n = self.n
        m = self.m
        if self.directed:
            for i in range(1, n + 1):
                mx = min(m - n + i, n - 1)
                mn = max(int(i != 1), m - (n - 1) * (n - i))
                cnt = randint(mn, mx)
                m -= cnt
                c1 = randint(max(int(i != 1), cnt - n + i), min(cnt, i - 1))
                f = list()
                if c1 != 0:
                    f += sample(range(1, i), c1)
                if cnt - c1 != 0:
                    f += sample(range(i + 1, n + 1), cnt - c1)
                for j in f:
                    self.add_edge(i, j)
        else:
            n = self.n
            m = self.m
            for i in range(2, n + 1):
                mx = min(m - n + i, i - 1)
                mn = max(1, m - int((n - 1) * n / 2) + int((i - 1) * i / 2))
                cnt = randint(mn, mx)
                m -= cnt
                f = sample(range(i - 1), cnt)
                for j in f:
                    self.add_edge(j + 1, i)



from random import randint, sample
from .base_graph import BaseGraph


class DAG(BaseGraph):
    def __init__(self, n: int, m: int, weighed: bool = False) -> None:
        """DAG initialization.

        Args:
            n (int): vertex number
            m (int): edge number
            weighed (bool, optional): weighted graph. Defaults to False.

        Raises:
            ValueError: Improper number of vertex or edge.
        """
        if n * (n - 1) < m * 2 or n < 0 or m < 0:
            raise ValueError("Improper number of vertex or edge.")
        super().__init__(n, m, True, weighed)

    def gen_edge(self):
        """DAG generation."""
        n = self.n
        m = self.m
        for i in range(2, n + 1):
            mx = min(m - n + i, i - 1)
            mn = max(1, m - int((n - 1) * n / 2) + int((i - 1) * i / 2))
            cnt = randint(mn, mx)
            m -= cnt
            f = sample(range(i - 1), cnt)
            for j in f:
                self.add_edge(j + 1, i)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['acmhelper',
 'acmhelper.cli',
 'acmhelper.cli.system',
 'acmhelper.cli.system.default',
 'acmhelper.gen',
 'acmhelper.gen.graph']

package_data = \
{'': ['*']}

install_requires = \
['loguru>=0.6.0,<0.7.0', 'typer[all]>=0.6.1,<0.7.0']

entry_points = \
{'console_scripts': ['helper = AcmHelper.cli.main:app']}

setup_kwargs = {
    'name': 'acmhelper',
    'version': '0.1.1',
    'description': 'The helper for Acmer',
    'long_description': '# AcmHelper\n\n本地环境下的 Polygon , 但不止于 Polygon.\n\n你可以\n\n- 快速创建具有合理结构的题目文件夹\n- 指定 `std` , `checker` , `validator` , `interactor` \n- 使用不同语言完成不同部分 (cpp/py)\n- 使用额外的程序来测试数据的质量\n- 使用预制的数据生成器快速生成具有某些特征的数据\n- 同时使用多种数据生成器 , 并可以指定每个程序所接受的生成器\n- 享受由 `rich` , `typer` 带来的美丽\n\nTODO\n\n- [ ] 使 `helper sys run` 可以执行 `save` 下的数据\n- [ ] 添加对 `Validator` 和 `Interactor` 的支持\n- [ ] 根据 `hash` 动态选择是否编译\n\n## 简易使用说明\n\n得益于 `typer` , 关于题目生成的功能都可以通过输入 `helper --help` 大致了解\n\n### 安装\n\n`pip install acmhelper`\n\n**你需要具有全局的`g++` , 默认使用 `-std=c++17` 编译** , 可在设置修改\n\n### 快速从数据文件得到渲染的图\n\n**如果想要使用图的渲染功能 , 你需要下载 Graphviz 的二进制文件并且将其加入 Path**\n\n对于形式为 \n\n```txt\nn m (optional)\nu1 v1 w1 (w1 is optional)\n...\nun vn wn\n```\n\n的数据 , 可以使用 `helper render` 来快速渲染\n\n首先创建文件 `test.in` , 写入一个不连通的 `DAG`\n\n```txt\n5 2\n1 2 3\n1 3 5\n1 5 -7\n```\n\n在该文件目录下输入 `helper render -dic test.in`\n\n查看 `test.png` , 应该如下所示\n\n![test](Assert/test.png)\n\n具体的设置请使用 `helper render --help` 查看\n\n### 创建一道题目(以 "输出一个绝对值小于输入数字绝对值的整数"为例(`int`范围内))\n\n首先新建文件夹 `Problem` , 然后在此文件夹下打开命令行 , 输入 `helper sys init` , 回答问题 , 完成初始化\n\n如果初始化正确的话你的目录结构应该向下面的一样(部分文件可能没有 , 这需要我们后续手动创建)\n\n```txt\n题目目录结构\n- Problem\n    - config.json\n    - std.cpp/py\n    - testlib.h (optional)\n    - checker.cpp/py (optional with "testlib.h")\n    - interactor.cpp (optional with "testlib.h") (Not implemented)\n    - validator.cpp (optional with "testlib.h") (Not implemented)\n    - generator\n        - make1.cpp\n        - make2.py\n    - accept\n        - ac1.cpp\n        - ac2.py\n    - wrong\n        - wa1.cpp\n        - wa2.py\n    - exec\n        - something executable...\n    - data\n        - auto\n            - in\n                - 1.make1.in\n                - 2.make2.in\n            - out\n                - 1.ac1.out\n                - 1.wa1.out\n        - save\n            - in\n                - 1.make1.in\n                - 2.make2.in\n            - out\n                - 1.ac1.out\n                - 1.wa1.out\n    - log\n        - 20220303.log\n    - temp\n        - someting temporary...\n    - output\n        - something...\n```\n\n然后打开 `std.cpp` , 写入以下代码\n\n```cpp\n#include <bits/stdc++.h>\nusing namespace std;\n\nint main() {\n    int a; \n    cin >> a; \n    cout << abs(a)/2 << \'\\n\'; \n    return 0;\n}\n```\n\n保存后 , 我们来写第一个数据生成器 `make1.py` , 它只用来生成大于 `0` 的数.\n\n在 `generator` 下创建文件 `make1.py` , 写入以下代码\n\n```py\nfrom random import randint\nprint(randint(1, 1000))\n```\n\n接下来只需要修改一些配置 , 就可以通过 `CLI` 来生成数据了\n\n打开文件 `config.json` , 将如下代码复制\n\n```json\n{\n    "gen_list": [\n        "make1"\n    ],\n    "accept_list": [],\n    "wrong_list": [],\n    "gen_link_code": {\n        "make1": [\n            "std"\n        ]\n    },\n    "gen_data_num": {\n        "make1": 10\n    },\n    "time_limit": 2,\n    "max_time_limit": 10,\n    "std": "std",\n    "checker": "checker",\n    "interactor": "",\n    "validator": "",\n    "gcc_version" 17\n}\n```\n\n这个文件描述的含义是 , 有一个生成器 `make1` , 其生成 10 组数据 , 生成的数据被用来运行 `std` , 没有额外的理论错误和理论正确的代码 , `Time_Limit_Exceed` 的上界是 `2s` , 程序被 kill 掉的上界是 `10s` , std 的辨识名称为 `std` , checker 的辨识名称为 `checker` , 没有使用 `interactor` 和 `validator` , 使用 `-std=c++17` 编译\n\n保存后 , 输入 `helper sys run` , 你应该能看到我们暂时性的成功\n\n接下来 , 我们来添加额外的生成器,测试代码和 `checker` 来完善这道题\n\n在 `wrong` 下创建 `wa1.cpp` 和 `wa2.py` , 然后写入下述代码\n\n```cpp\n#include <bits/stdc++.h>\nusing namespace std;\n\nint main() {\n    int a; \n    cin >> a; \n    cout << a-1 << \'\\n\'; \n    return 0;\n}\n```\n\n\n\n```py\nn = int(input())\nprint(n + 1)\n```\n\n\n\n在 `accept` 下创建 `ac1.cpp` 写入如下代码\n\n```cpp\n#include <bits/stdc++.h>\nusing namespace std;\n\nint main() {\n    int a; \n    cin >> a; \n    if(a >= 0) { cout << a-1 << \'\\n\'; }\n    else { cout << a+1 << \'\\n\'; }\n    return 0;\n}\n```\n\n可以发现 , `wrong` 下的两个代码分别会在 `a<0` 和 `a>0` 时出错 , 所以我们再新建一个生成器\n\n在 `generator` 下创建 `make2.py` 写入如下代码 , 注意负号\n\n```py\nfrom random import randint\nprint(-randint(1, 1000))\n```\n\n可以发现这道题需要 `checker` , 打开 `checker.cpp` , 写入如下代码\n\n```cpp\n#include <bits/stdc++.h>\n#include "testlib.h"\nusing namespace std;\n\nint main(int argc,char** argv) {\n    registerTestlibCmd(argc,argv); // Required\n    int n = inf.readInt();\n    int m = ouf.readInt();\n    if(abs(m) < abs(n)) { quitf(_ok,"Correct!"); }\n    else { quitf(_wa,"Wrong Answer!"); }\n}\n```\n\n最后我们再修改一下 `config.json`\n\n```json\n{\n    "gen_list": [\n        "make1",\n        "make2"\n    ],\n    "accept_list": [\n        "ac1"\n    ],\n    "wrong_list": [\n        "wa1",\n        "wa2"\n    ],\n    "gen_link_code": {\n        "make1": [\n            "std",\n            "ac1",\n            "wa1",\n            "wa2"\n        ],\n        "make2": [\n            "std",\n            "ac1",\n            "wa1",\n            "wa2"\n        ]\n    },\n    "gen_data_num": {\n        "make1": 10,\n        "make2": 10\n    },\n    "time_limit": 2,\n    "max_time_limit": 10,\n    "std": "std",\n    "checker": "checker",\n    "interactor": "",\n    "validator": "",\n    "gcc_version": 17\n}\n```\n\n最后输入 `helper sys run` , 完成了.\n\n如果字体和终端合适 , 你应该会看到像这样的东西\n\n![result](Assert/table.png)\n\n不出意料地 , 两个不对的程序在合适的地方不对了.\n\n作为结尾 , 我们来使用 `CLI` 打包数据以及 `checker`\n\n输入 `helper sys add 1 2 3 4 5 6 7 8 9 10` , 这样我们就将前10组输入数据和 `std` 的输出数据从 `auto` 移动到了 `save` 下 , 值得注意的是 , 数据的编号会自动地递增 , 所以不用担心数据覆盖问题\n\n然后输入 `helper sys output checker.cc` , 查看 `output` 文件夹下 , 你应该可以看到名为 `data.zip` 的文件 , 其中的所有文件都被合适的重命名了 , 特殊的 , `checker` 被重命名为了 `checker.cc`\n\n更细节的使用请参考 `helper sys --help` , `helper sys add --help` 等',
    'author': 'CherryGS',
    'author_email': 'tickten@outlook.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
